/////////////////////////////////////////

#include <Wire.h>

// DS1307 clock registers
#define DS1307      0xD0 >> 1                   // shift required by Wire.h (silly...)
#define R_SECS      0
#define R_MINS      1
#define R_HRS       2
#define R_WKDAY     3
#define R_DATE      4
#define R_MONTH     5
#define R_YEAR      6
#define R_SQW       7

struct DS1307_Time
{
    byte second; 
    byte minute;
    byte hour;
    byte wkDay;
    byte day;
    byte month;
    byte year;
    byte ctrl;
};

DS1307_Time time;

//keypad debounce parameter
#define DEBOUNCE_MAX 15
#define DEBOUNCE_ON  10
#define DEBOUNCE_OFF 3 

#define NUM_KEYS 4

// keypad number
#define UP_KEY 1
#define DOWN_KEY 0
#define OK_KEY 2
#define CANCEL_KEY 3

// adc preset value, represent top value,incl. noise & margin,that the adc reads, when a key is pressed
// set noise & margin = 30 (0.15V@5V)
int  adc_key_val[NUM_KEYS] ={0, 210, 440, 660};

long previousMillis = 0;        // will store last time LED was updated

struct Keypad
{
    byte button_count; // debounce counters
    byte button_status; // button status - pressed/released
    byte button_flag; // button on flags for user program 

    byte button_long_pressed;
    int button_count_long;   
};

Keypad key[NUM_KEYS];

unsigned char SevenSegmentDigi[] = { 8, 9, 10, 11};

unsigned char number_table[10] = {
    0b11000000 , //0
    0b11111001 , //1
    0b10100100 , //2
    0b10110000 , //3
    0b10011001 , //4
    0b10010010 , //5
    0b10000010 , //6
    0b11111000 , //7
    0b10000000 , //8
    0b10010000   //9
};


void setup()
{
   Wire.begin();                                 // Initial I2C
   Serial.begin(19200);
    
   init_keypad();   
   init_SevenSegment();
}

void loop()
{
  
  byte i;
    for(i=0; i<NUM_KEYS; i++){
                 
        if(key[i].button_flag !=0)
        {            
           key[i].button_flag=0;  // reset button flag
           
 	    switch(i)
            {
      	    case UP_KEY:
              Serial.print(key[i].button_long_pressed,DEC);
              Serial.println(",UP");             
              break;
    	    case DOWN_KEY:
              Serial.print(key[i].button_long_pressed,DEC);    
              Serial.println(",DOWN");
	      break;
            case OK_KEY:        
              Serial.print(key[i].button_long_pressed,DEC);            
              Serial.println(",OK");  
                        
              break;   
            case CANCEL_KEY:
              Serial.print(key[i].button_long_pressed,DEC);            
              Serial.println(",CANCEL");
              break;
	    }

	}
      
  }
  
  if (key[OK_KEY].button_long_pressed ==1) setupTime();

  if (millis() - previousMillis > 1000) {
    
    previousMillis = millis();   // remember the last time we blinked the LED    
    displayClock();

  }
  

}

void displayClock()
{
    getClock();                                   // Read RTC Clock

    Serial.print(bcd2Dec(time.hour),DEC);
    Serial.print(":");
    Serial.print(bcd2Dec(time.minute),DEC);
    Serial.print(":");
    Serial.print(bcd2Dec(time.second),DEC);    
    Serial.println();
}


void setupTime()
{
  byte i,val;
  byte STATE;
  
  STATE=0;
  
  val = bcd2Dec(time.minute);
  
  while(STATE == 0)
  {    
    for(i=0; i<NUM_KEYS; i++){
                 
        if(key[i].button_flag !=0)
        {            
           key[i].button_flag=0;  // reset button flag
           
 	    switch(i)
            {
      	    case UP_KEY:
              val++;
              break;
    	    case DOWN_KEY:
              val--;
	      break;
            case OK_KEY:        
              //return;
              STATE = 1;
              break;   
            case CANCEL_KEY:
              Serial.println("CANCEL");
              return;
	    }

	}
      
  }
   
  Serial.println(val,DEC);

  }
  
  time.minute = Dec2bcd(val);
  
  setClock();
  
}
/// function for ds1307 ///////
void setClock()
{
    Wire.beginTransmission(DS1307);
    Wire.send(R_SECS);
    Wire.send(time.second);
    Wire.send(time.minute);
    Wire.send(time.hour);
    Wire.send(time.wkDay);
    Wire.send(time.day);
    Wire.send(time.month);
    Wire.send(time.year);
    Wire.send(time.ctrl);
    Wire.endTransmission();
}


void getClock()
{
    Wire.beginTransmission(DS1307);
    Wire.send(R_SECS);
    Wire.endTransmission();
    
    Wire.requestFrom(DS1307, 8);
    time.second = Wire.receive();
    time.minute = Wire.receive();
    time.hour   = Wire.receive();
    time.wkDay  = Wire.receive();
    time.day    = Wire.receive();
    time.month  = Wire.receive();
    time.year   = Wire.receive();
    time.ctrl   = Wire.receive();
}


byte bcd2Dec(byte bcdVal)
{
 return bcdVal / 16 * 10 + bcdVal % 16;
}

byte Dec2bcd(byte DecVal)
{
  return DecVal/10*16 + DecVal % 10;
}

/// for seven segment
void initSevenSegment(void)
{
  for(int i=0; i<13; i++)
       pinMode(i, OUTPUT);
}

void sevenSegment( byte val)
{
  //for (int i=0; i<8; i++)
  //   digitalWrite(SevenSegmentpins[i], font_table[val][i]); 
  
  PORTD = number_table[val];  
       
}

void ControlSegmentDigi( byte val)
{
  for (int i=0; i<4 ;i++)
     if (val != i)
       digitalWrite(SevenSegmentDigi[i],HIGH);  
  
  digitalWrite(SevenSegmentDigi[val],LOW);    
  delay(2);
}



// for keypad

void init_keypad(){
  
   for(byte i=0; i<NUM_KEYS; i++){
     key[i].button_count_long = 0;
     key[i].button_count=0;
     key[i].button_status=0;
     key[i].button_flag=0;
   }
  
  // Setup timer2 -- Prescaler/256
  TCCR2A &= ~((1<<WGM21) | (1<<WGM20));
  TCCR2B &= ~(1<<WGM22);
  TCCR2B = (1<<CS22)|(1<<CS21);      
  
   ASSR |=(0<<AS2);

  // Use normal mode  
   TCCR2A =0;    
   
  //Timer2 Overflow Interrupt Enable  
   TIMSK2 |= (0<<OCIE2A);
   TCNT2=0x6;  // counting starts from 6;  
   TIMSK2 = (1<<TOIE2);    
                      
   SREG|=1<<SREG_I; 
}

         
// waiting for center key press
void waitfor_OKkey(){
  byte i;
  byte keyOK = 0xFF;
  
  while (keyOK!= OK_KEY){
  
    for(i=0; i<NUM_KEYS; i++){
       
      if(key[i].button_flag !=0){
           key[i].button_flag = 0;  // reset button flag
           if(i== OK_KEY) keyOK=OK_KEY;
        }
        
     }
   }		
}


// The followinging are interrupt-driven keypad reading 
//  which includes DEBOUNCE ON/OFF mechanism, and continuous pressing detection

// Convert ADC value to key number
char get_key(unsigned int input)
{
  char k;
  
  for (k = 0; k < NUM_KEYS; k++)
  {
	if (input <= adc_key_val[k])
	{
          return k;
        }
  }

  if (k >= NUM_KEYS)  k = -1;     // No valid key pressed
    
  return k;
  
}

void update_adc_key(){
  int adc_key_in;
  char key_in;
  byte i;
  
  adc_key_in = analogRead(0);
  key_in = get_key(adc_key_in);
  for(i=0; i<NUM_KEYS; i++)
  {
    if(key_in==i)  //one key is pressed 
    {          
      if(key[i].button_count < DEBOUNCE_MAX)
      {
        key[i].button_count++;
        
        if( key[i].button_count>DEBOUNCE_ON) 
        {          
          if( key[i].button_status == 0) 
          { 
            key[i].button_flag = 1;
            key[i].button_status = 1; //button debounced to 'pressed' status
          }		            
        }
      }
      else
      {
        key[i].button_count_long++;
        if (key[i].button_count_long > 255)
        {
          key[i].button_flag=1;
          key[i].button_status=1;
          key[i].button_long_pressed=1;
          key[i].button_count_long=255;
        }
      }     
    }
    else // no button pressed 
    {
      
      if (key[i].button_count >0) {  
        
          key[i].button_flag = 0;	
          key[i].button_count--;
          
        if( key[i].button_count < DEBOUNCE_OFF) {
          key[i].button_status = 0;   //button debounced to 'released' status 
          key[i].button_long_pressed = 0;
          key[i].button_count_long = 0;
          
        }
        
      }
    }    
  }
}

// Timer2 interrupt routine -
// 1/(160000000/256/(256-6)) = 4ms interval

ISR(TIMER2_OVF_vect) {  
  TCNT2  = 6;
  update_adc_key();
}

