// Analog Keypad

//keypad debounce parameter
#define DEBOUNCE_MAX 15
#define DEBOUNCE_ON  10
#define DEBOUNCE_OFF 3 

void init_keypad()
{
  
   for(byte i=0; i<NUM_KEYS; i++){
     key[i].button_count_long = 0;
     key[i].button_count=0;
     key[i].button_status=0;
     key[i].button_flag=0;
   }
  
  // Setup timer2 -- Prescaler/256
  TCCR2A &= ~((1<<WGM21) | (1<<WGM20));
  TCCR2B &= ~(1<<WGM22);
  TCCR2B = (1<<CS22)|(1<<CS21);      
  
   ASSR |=(0<<AS2);

  // Use normal mode  
   TCCR2A =0;    
   
  //Timer2 Overflow Interrupt Enable  
   TIMSK2 |= (0<<OCIE2A);
   TCNT2=0x6;  // counting starts from 6;  
   TIMSK2 = (1<<TOIE2);    
                      
   SREG|=1<<SREG_I; 
}

         
// waiting for center key press
void waitfor_OKkey(){
  byte i;
  byte keyOK = 0xFF;
  
  while (keyOK!= OK_KEY){
  
    for(i=0; i<NUM_KEYS; i++){
       
      if(key[i].button_flag !=0){
           key[i].button_flag = 0;  // reset button flag
           if(i== OK_KEY) keyOK=OK_KEY;
        }
        
     }
   }		
}

// The followinging are interrupt-driven keypad reading 
//  which includes DEBOUNCE ON/OFF mechanism, and continuous pressing detection

// Convert ADC value to key number
char get_key(unsigned int input)
{
  char k;
  
  for (k = 0; k < NUM_KEYS; k++)
  {
	if (input <= adc_key_val[k])
	{
          return k;
        }
  }

  if (k >= NUM_KEYS)  k = -1;     // No valid key pressed
    
  return k;
  
}

void update_adc_key(){
  int adc_key_in;
  char key_in;
  byte i;
  
  adc_key_in = analogRead(0);
  key_in = get_key(adc_key_in);
  for(i=0; i<NUM_KEYS; i++)
  {
    if(key_in==i)  //one key is pressed 
    {          
      if(key[i].button_count < DEBOUNCE_MAX)
      {
        key[i].button_count++;
        
        if( key[i].button_count > DEBOUNCE_ON) 
        {          
          if( key[i].button_status == 0) 
          { 
            key[i].button_flag = 1;
            key[i].button_status = 1; //button debounced to 'pressed' status
          }		            
        }
      }
      else
      {
        key[i].button_count_long++;
        if (key[i].button_count_long > 100)
        {
          key[i].button_flag = 0;
          key[i].button_status = 1;
          key[i].button_long_pressed = 1;
          key[i].button_count_long = 100;
        }
      }     
    }
    else // no button pressed 
    {
      
      if (key[i].button_count >0) {  
        
          key[i].button_flag = 0;	
          key[i].button_count--;
          
        if( key[i].button_count < DEBOUNCE_OFF) {
          key[i].button_status = 0;   //button debounced to 'released' status 
          key[i].button_long_pressed = 0;
          key[i].button_count_long = 0;
          
        }
        
      }
    }    
  }
}

// Timer2 interrupt routine -
// 1/(160000000/256/(256-6)) = 4ms interval

ISR(TIMER2_OVF_vect) {  
  TCNT2  = 6;
  update_adc_key();
}
